@@ -8,7 +8,7 @@
     'name_es_ES': 'Estado de cuentas',
     'name_fr_FR': 'Relevé comptable',
     'name_nl_NL': 'Bankafschriften',
-    'version': '2.0.2',
+    'version': '2.2.2',
     'author': 'B2CK',
     'email': 'info@b2ck.com',
     'website': 'http://www.tryton.org/',
@@ -52,11 +52,13 @@
         'invoice.xml',
         ],
     'translation': [
-        'bg_BG.csv',
-        'de_DE.csv',
-        'es_CO.csv',
-        'es_ES.csv',
-        'fr_FR.csv',
-        'nl_NL.csv',
+        'locale/bg_BG.po',
+        'locale/cs_CZ.po',
+        'locale/de_DE.po',
+        'locale/es_CO.po',
+        'locale/es_ES.po',
+        'locale/fr_FR.po',
+        'locale/nl_NL.po',
+        'locale/ru_RU.po',
     ],
 }
