@@ -1,13 +1,14 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from decimal import Decimal
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
-from trytond.pyson import Not, Equal, Eval, Or, Bool, Get, If, In
+from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.backend import TableHandler
+from trytond.pool import Pool
 
-_STATES = {'readonly': Not(Equal(Eval('state'), 'draft'))}
+_STATES = {'readonly': Eval('state') != 'draft'}
+_DEPENDS = ['state']
 
 
 class Statement(ModelWorkflow, ModelSQL, ModelView):
@@ -17,31 +18,40 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
 
     company = fields.Many2One('company.company', 'Company', required=True,
         select=1, states=_STATES, domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
-                Get(Eval('context', {}), 'company', 0)),
-        ])
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
+                Eval('context', {}).get('company', 0)),
+            ],
+        depends=_DEPENDS)
     journal = fields.Many2One('account.statement.journal', 'Journal', required=True,
         domain=[
-            ('company', '=', Eval('company')),
-        ],
+            ('company', '=', Eval('context', {}).get('company', 0)),
+            ],
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines'))),
-        }, on_change=['journal'], select=1)
+            'readonly': (Eval('state') != 'draft') | Eval('lines'),
+            },
+        on_change=['journal', 'state', 'lines'], select=1,
+        depends=['state', 'lines'])
     currency_digits = fields.Function(fields.Integer('Currency Digits',
         on_change_with=['journal']), 'get_currency_digits')
-    date = fields.Date('Date', required=True, states=_STATES, select=1)
+    date = fields.Date('Date', required=True, states=_STATES, depends=_DEPENDS,
+        select=1)
     start_balance = fields.Numeric('Start Balance',
-            digits=(16, Eval('currency_digits', 2)),
-            states=_STATES, depends=['currency_digits'])
+        digits=(16, Eval('currency_digits', 2)),
+        states=_STATES, depends=['state', 'currency_digits'])
     end_balance = fields.Numeric('End Balance',
+        digits=(16, Eval('currency_digits', 2)),
+        states=_STATES, depends=['state', 'currency_digits'])
+    balance = fields.Function(
+        fields.Numeric('Balance',
             digits=(16, Eval('currency_digits', 2)),
-            states=_STATES, depends=['currency_digits'])
+            on_change_with=['start_balance', 'end_balance'],
+            depends=['currency_digits']),
+        'get_balance')
     lines = fields.One2Many('account.statement.line', 'statement',
-            'Transactions', states={
-                'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                    Not(Bool(Eval('journal')))),
-            }, on_change=['lines', 'journal'])
+        'Transactions', states={
+            'readonly': (Eval('state') != 'draft') | ~Eval('journal'),
+            }, on_change=['lines', 'journal'],
+        depends=['state', 'journal'])
     state = fields.Selection([
         ('draft', 'Draft'),
         ('validated', 'Validated'),
@@ -92,11 +102,11 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         return 'draft'
 
     def default_date(self):
-        date_obj = self.pool.get('ir.date')
+        date_obj = Pool().get('ir.date')
         return date_obj.today()
 
     def default_currency_digits(self):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
         if Transaction().context.get('company'):
             company = company_obj.browse(Transaction().context['company'])
             return company.currency.digits
@@ -120,7 +130,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def on_change_with_currency_digits(self, vals):
-        journal_obj = self.pool.get('account.statement.journal')
+        journal_obj = Pool().get('account.statement.journal')
         if vals.get('journal'):
             journal = journal_obj.browse(vals['journal'])
             return journal.currency.digits
@@ -133,7 +143,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         return res
 
     def get_rec_name(self, ids, name):
-        lang_obj = self.pool.get('ir.lang')
+        lang_obj = Pool().get('ir.lang')
 
         if not ids:
             return {}
@@ -189,10 +199,21 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                 res[statement.id] += line.amount
         return res
 
+    def get_balance(self, ids, name):
+        return dict((s.id, s.end_balance - s.start_balance)
+            for s in self.browse(ids))
+
+    def on_change_with_balance(self, values):
+        if not set(('end_balance', 'start_balance')) <= set(values):
+            return 0
+        else:
+            return values['end_balance'] - values['start_balance']
+
     def on_change_lines(self, values):
-        invoice_obj = self.pool.get('account.invoice')
-        journal_obj = self.pool.get('account.statement.journal')
-        currency_obj = self.pool.get('currency.currency')
+        pool = Pool()
+        invoice_obj = pool.get('account.invoice')
+        journal_obj = pool.get('account.statement.journal')
+        currency_obj = pool.get('currency.currency')
         res = {
             'lines': {},
         }
@@ -203,7 +224,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                 if line['invoice']:
                     invoice_ids.add(line['invoice'])
             invoice_id2amount_to_pay = {}
-            for invoice in invoice_obj.browse(invoice_ids):
+            for invoice in invoice_obj.browse(list(invoice_ids)):
                 with Transaction().set_context(date=invoice.currency_date):
                     invoice_id2amount_to_pay[invoice.id] = currency_obj.compute(
                         invoice.currency.id, invoice.amount_to_pay,
@@ -239,11 +260,9 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                             amount_to_pay - abs(line['amount'])
         return res
 
-    def set_state_validated(self, statement_id):
-        statement_line_obj = self.pool.get('account.statement.line')
-        lang_obj = self.pool.get('ir.lang')
-
-        statement = self.browse(statement_id)
+    def wkf_validated(self, statement):
+        statement_line_obj = Pool().get('account.statement.line')
+        lang_obj = Pool().get('ir.lang')
 
         computed_end_balance = statement.start_balance
         for line in statement.lines:
@@ -263,25 +282,23 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
             self.raise_user_error('wrong_end_balance', error_args=(amount,))
         for line in statement.lines:
             statement_line_obj.create_move(line)
-        self.write(statement_id, {
+        self.write(statement.id, {
             'state':'validated',
             })
 
-    def set_state_posted(self, statement_id):
-        statement_line_obj = self.pool.get('account.statement.line')
+    def wkf_posted(self, statement):
+        statement_line_obj = Pool().get('account.statement.line')
 
-        statement = self.browse(statement_id)
         statement_line_obj.post_move(statement.lines)
-        self.write(statement_id, {
+        self.write(statement.id, {
             'state':'posted',
             })
 
-    def set_state_cancel(self, statement_id):
-        statement_line_obj = self.pool.get('account.statement.line')
+    def wkf_cancel(self, statement):
+        statement_line_obj = Pool().get('account.statement.line')
 
-        statement = self.browse(statement_id)
         statement_line_obj.delete_move(statement.lines)
-        self.write(statement_id, {
+        self.write(statement.id, {
             'state':'cancel',
             })
 
@@ -304,29 +321,29 @@ class Line(ModelSQL, ModelView):
             required=True, ondelete='CASCADE')
     date = fields.Date('Date', required=True)
     amount = fields.Numeric('Amount', required=True,
-            digits=(16, Get(Eval('_parent_statement', {}),
-                'currency_digits', 2)),
-            on_change=['amount', 'party', 'account', 'invoice',
-                '_parent_statement.journal'])
+        digits=(16, Eval('_parent_statement', {}).get( 'currency_digits', 2)),
+        on_change=['amount', 'party', 'account', 'invoice',
+            '_parent_statement.journal'])
     party = fields.Many2One('party.party', 'Party',
             on_change=['amount', 'party', 'invoice'])
     account = fields.Many2One('account.account', 'Account', required=True,
-            on_change=['account', 'invoice'], domain=[
-                ('kind', '!=', 'view'),
-                ('company', '=', Get(Eval('_parent_statement', {}), 'company')),
+        on_change=['account', 'invoice'], domain=[
+            ('kind', '!=', 'view'),
+            ('company', '=', Eval('_parent_statement', {}).get('company', 0)),
             ])
     description = fields.Char('Description')
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
     invoice = fields.Many2One('account.invoice', 'Invoice',
-            domain=[
-                ('party', '=', Eval('party')),
-                ('account', '=', Eval('account')),
-                ('state', '=', If(Equal(Get(Eval('_parent_statement', {}),
-                    'state'), 'draft'), 'open', '')),
+        domain=[
+            ('party', '=', Eval('party')),
+            ('account', '=', Eval('account')),
+            ('state', '=', If(Eval('_parent_statement', {}).get('state')
+                    == 'draft', 'open', '')),
             ],
-            states={
-                'readonly': Not(Bool(Eval('amount'))),
-            })
+        states={
+            'readonly': ~Eval('amount'),
+            },
+        depends=['party', 'account', 'amount'])
 
     def __init__(self):
         super(Line, self).__init__()
@@ -340,8 +357,8 @@ class Line(ModelSQL, ModelView):
             })
 
     def on_change_party(self, value):
-        party_obj = self.pool.get('party.party')
-        invoice_obj = self.pool.get('account.invoice')
+        party_obj = Pool().get('party.party')
+        invoice_obj = Pool().get('account.invoice')
         res = {}
 
         if value.get('party'):
@@ -364,10 +381,11 @@ class Line(ModelSQL, ModelView):
         return res
 
     def on_change_amount(self, value):
-        party_obj = self.pool.get('party.party')
-        invoice_obj = self.pool.get('account.invoice')
-        journal_obj = self.pool.get('account.statement.journal')
-        currency_obj = self.pool.get('currency.currency')
+        pool = Pool()
+        party_obj = pool.get('party.party')
+        invoice_obj = pool.get('account.invoice')
+        journal_obj = pool.get('account.statement.journal')
+        currency_obj = pool.get('currency.currency')
         res = {}
 
         if value.get('party'):
@@ -397,7 +415,7 @@ class Line(ModelSQL, ModelView):
         return res
 
     def on_change_account(self, value):
-        invoice_obj = self.pool.get('account.invoice')
+        invoice_obj = Pool().get('account.invoice')
         res = {}
 
         if value.get('invoice'):
@@ -416,12 +434,13 @@ class Line(ModelSQL, ModelView):
         :param line: a BrowseRecord of the line
         :return: the move id
         '''
-        move_obj = self.pool.get('account.move')
-        period_obj = self.pool.get('account.period')
-        invoice_obj = self.pool.get('account.invoice')
-        currency_obj = self.pool.get('currency.currency')
-        move_line_obj = self.pool.get('account.move.line')
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        move_obj = pool.get('account.move')
+        period_obj = pool.get('account.period')
+        invoice_obj = pool.get('account.invoice')
+        currency_obj = pool.get('currency.currency')
+        move_line_obj = pool.get('account.move.line')
+        lang_obj = pool.get('ir.lang')
 
         period_id = period_obj.find(line.statement.company.id,
                 date=line.date)
@@ -482,11 +501,11 @@ class Line(ModelSQL, ModelView):
         return move_id
 
     def post_move(self, lines):
-        move_obj = self.pool.get('account.move')
+        move_obj = Pool().get('account.move')
         move_obj.post([l.move.id for l in lines if l.move])
 
     def delete_move(self, lines):
-        move_obj = self.pool.get('account.move')
+        move_obj = Pool().get('account.move')
         move_obj.delete([l.move.id for l in lines if l.move])
 
     def _get_move_lines(self, statement_line):
@@ -496,7 +515,7 @@ class Line(ModelSQL, ModelView):
         :param statement_line: a BrowseRecord of the statement line
         :return: a list of dictionary of move line values
         '''
-        currency_obj = self.pool.get('currency.currency')
+        currency_obj = Pool().get('currency.currency')
         zero = Decimal("0.0")
         amount = currency_obj.compute(
             statement_line.statement.journal.currency, statement_line.amount,
